# 24 "cil/src/logic/logic_lexer.mll"
 

  open Logic_parser
  open Lexing
  open Logic_ptree

  type state = Normal | Test

  let state_stack = Stack.create ()

  let () = Stack.push Normal state_stack

  let get_state () = try Stack.top state_stack with Stack.Empty -> Normal

  let pop_state () = try ignore (Stack.pop state_stack) with Stack.Empty -> ()

  exception Error of (int * int) * string

  let loc lexbuf = (lexeme_start lexbuf, lexeme_end lexbuf)

  let lex_error lexbuf s =
    raise (Error (loc lexbuf, "lexical error, " ^ s))

  let find_utf8 =
    let h = Hashtbl.create 97 in
    List.iter (fun (i,t) -> Hashtbl.add h i t)
      [ Utf8_logic.forall, FORALL;
        Utf8_logic.exists, EXISTS;
        Utf8_logic.eq, EQ;
        Utf8_logic.neq, NE;
        Utf8_logic.le, LE;
        Utf8_logic.ge, GE;
        Utf8_logic.implies,IMPLIES;
        Utf8_logic.iff, IFF;
        Utf8_logic.conj, AND;
        Utf8_logic.disj, OR;
        Utf8_logic.neg, NOT;
        Utf8_logic.x_or, HATHAT;
      ];

    fun s -> try Hashtbl.find h s
    with Not_found -> IDENTIFIER s

  let identifier =
    let all_kw = Hashtbl.create 37 in
    let c_kw = Hashtbl.create 37 in
    List.iter
      (fun (i,t,flag) ->
         Hashtbl.add all_kw i t;
         if flag then Hashtbl.add c_kw i t
      )
      [
        "assert", ASSERT, false;
        "assigns", ASSIGNS, false;
        "assumes", ASSUMES, false;
        "axiom", AXIOM, false;
        "axiomatic", AXIOMATIC, false;
        "behavior", BEHAVIOR, false;
        "behaviors", BEHAVIORS, false;
	"case", CASE, true;
        "char", CHAR, true;
        "complete", COMPLETE, false;
        "decreases", DECREASES, false;
        "disjoint", DISJOINT, false;
        "double", DOUBLE, true;
        "else", ELSE, true;
        "ensures", ENSURES, false ;
        "enum", ENUM, true;
        "float", FLOAT, true;
        "for", FOR, true;
        "if", IF, true;
	"inductive", INDUCTIVE, false;
        "int", INT, true;
        "integer", INTEGER, true;
        "invariant", INVARIANT, false;
        "global",    GLOBAL, false;
        "label", LABEL, false;
        "lemma", LEMMA, false;
        "logic", LOGIC, false;
        "long", LONG, true;
        "loop", LOOP, false;
        "pragma", PRAGMA, false;
        "predicate", PREDICATE, false;
        "reads", READS, false;
        "real", REAL, true;
        "requires", REQUIRES, false;
        "short", SHORT, true;
        "signed", SIGNED, true;
        "sizeof", SIZEOF, true;
        "slice", SLICE, false;
	"impact", IMPACT, false;
        "struct", STRUCT, true;
        "terminates", TERMINATES, false;
        "type", TYPE, false;
        "union", UNION, true;
        "unsigned", UNSIGNED, true;
        "variant", VARIANT, false;
        "void", VOID, true;
      ];
    fun s ->
      try
        Hashtbl.find (if Logic_const.is_kw_c_mode () then c_kw else all_kw) s
      with Not_found ->
        try
          if Hashtbl.find Logic_const.typenames s then
            TYPENAME s
          else IDENTIFIER s
        with Not_found -> IDENTIFIER s

  let bs_identifier =
    let h = Hashtbl.create 97 in
    List.iter (fun (i,t) -> Hashtbl.add h i t)
      [
        "\\at", AT;
        "\\base_addr", BASE_ADDR;
        "\\block_length", BLOCK_LENGTH;
        "\\empty", EMPTY;
        "\\exists", EXISTS;
        "\\false", FALSE;
        "\\forall", FORALL;
        "\\fresh", FRESH;
        "\\from", FROM;
        "\\inter", INTER;
        "\\lambda", LAMBDA;
        "\\max", IDENTIFIER "\\max";
        "\\min", IDENTIFIER "\\min";
        "\\nothing", NOTHING;
        "\\null", NULL;
        "\\numof", IDENTIFIER "\\numof";
        "\\old", OLD;
        "\\product", IDENTIFIER "\\product";
        "\\result", RESULT;
        "\\separated", SEPARATED;
        "\\sum", IDENTIFIER "\\sum";
        "\\true", TRUE;
        "\\type", BSTYPE;
        "\\typeof", TYPEOF;
        "\\union", BSUNION;
        "\\valid", VALID;
        "\\valid_index", VALID_INDEX;
        "\\valid_range", VALID_RANGE;
      ];
    fun lexbuf ->
      let s = lexeme lexbuf in
      try Hashtbl.find h s
      with Not_found ->
        lex_error lexbuf ("illegal escape sequence " ^ s)


  let int_of_digit chr =
    match chr with
        '0'..'9' -> (Char.code chr) - (Char.code '0')
      | 'a'..'f' -> (Char.code chr) - (Char.code 'a') + 10
      | 'A'..'F' -> (Char.code chr) - (Char.code 'A') + 10
      | _ -> assert false

  (* Update lexer buffer. *)
  let update_newline_loc lexbuf =
    let pos = lexbuf.Lexing.lex_curr_p in
    lexbuf.Lexing.lex_curr_p <-
      { pos with
	Lexing.pos_lnum = pos.Lexing.pos_lnum + 1;
	Lexing.pos_bol = pos.Lexing.pos_cnum;
      }

  (* Update lexer buffer. *)
  let update_line_loc lexbuf line absolute chars =
    let pos = lexbuf.Lexing.lex_curr_p in
    lexbuf.Lexing.lex_curr_p <-
      { pos with
	Lexing.pos_lnum = if absolute then line else pos.Lexing.pos_lnum + line;
	Lexing.pos_bol = pos.Lexing.pos_cnum - chars;
      }

  (* Update lexer buffer. *)
  let update_file_loc lexbuf file =
    let pos = lexbuf.Lexing.lex_curr_p in
    lexbuf.Lexing.lex_curr_p <-
      { pos with
	Lexing.pos_fname = file;
      }

  (* Update lexer buffer. *)
  let update_bol_loc lexbuf bol =
    let pos = lexbuf.Lexing.lex_curr_p in
    lexbuf.Lexing.lex_curr_p <-
      { pos with
	Lexing.pos_bol = bol;
      }

# 193 "cil/src/logic/logic_lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\195\255\196\255\000\000\202\255\203\255\204\255\205\255\
    \206\255\207\255\196\000\215\255\217\255\165\000\222\255\223\255\
    \202\000\230\255\231\255\199\000\233\255\234\255\201\000\139\000\
    \227\000\208\000\206\000\240\255\004\001\225\000\006\001\254\000\
    \036\001\109\001\198\001\000\002\236\000\254\255\030\001\091\001\
    \252\255\253\255\075\002\092\001\127\002\128\002\129\002\159\002\
    \246\255\241\255\153\002\182\002\208\000\214\002\033\003\088\003\
    \211\000\212\000\224\002\079\001\245\255\242\255\243\255\253\002\
    \158\003\244\255\061\003\127\003\192\002\168\003\197\003\239\000\
    \229\003\002\004\012\004\239\003\225\255\224\000\239\255\199\255\
    \201\255\228\000\228\000\238\255\237\255\236\255\208\255\232\255\
    \200\255\228\255\220\255\198\255\211\255\200\000\251\255\044\004\
    \097\001\117\004\140\004\190\001\202\004\186\000\073\004\255\255\
    \202\000\212\000\137\003\224\001\046\003\093\001";
  Lexing.lex_backtrk = 
   "\255\255\255\255\255\255\058\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\042\000\255\255\255\255\036\000\255\255\255\255\
    \026\000\255\255\255\255\031\000\255\255\255\255\020\000\037\000\
    \034\000\041\000\046\000\255\255\060\000\028\000\060\000\008\000\
    \008\000\005\000\005\000\060\000\039\000\255\255\000\000\255\255\
    \255\255\255\255\004\000\255\255\255\255\255\255\255\255\009\000\
    \255\255\255\255\012\000\255\255\008\000\007\000\255\255\006\000\
    \006\000\007\000\010\000\255\255\255\255\255\255\255\255\255\255\
    \011\000\255\255\255\255\011\000\255\255\012\000\255\255\029\000\
    \011\000\255\255\011\000\255\255\255\255\045\000\255\255\255\255\
    \255\255\043\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\004\000\
    \001\000\255\255\000\000\001\000\255\255\004\000\002\000\255\255\
    \255\255\255\255\255\255\003\000\255\255\255\255";
  Lexing.lex_default = 
   "\003\000\000\000\000\000\255\255\000\000\000\000\000\000\000\000\
    \000\000\000\000\255\255\000\000\000\000\255\255\000\000\000\000\
    \255\255\000\000\000\000\255\255\000\000\000\000\255\255\255\255\
    \255\255\255\255\255\255\000\000\043\000\255\255\045\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\000\000\255\255\039\000\
    \000\000\000\000\255\255\043\000\045\000\045\000\045\000\045\000\
    \000\000\000\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\000\000\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\255\255\000\000\000\000\
    \000\000\255\255\255\255\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\094\000\000\000\255\255\
    \255\255\255\255\255\255\255\255\094\000\255\255\255\255\000\000\
    \255\255\255\255\040\000\108\000\108\000\041\000";
  Lexing.lex_trans = 
   "\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
    \001\000\038\000\037\000\001\000\038\000\038\000\001\000\001\000\
    \001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
    \001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
    \038\000\022\000\028\000\027\000\021\000\011\000\024\000\030\000\
    \009\000\008\000\014\000\015\000\020\000\019\000\029\000\036\000\
    \032\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\016\000\017\000\025\000\026\000\010\000\018\000\
    \038\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\034\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\005\000\035\000\004\000\013\000\033\000\
    \001\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\007\000\023\000\006\000\012\000\001\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\001\000\
    \002\000\092\000\091\000\090\000\089\000\087\000\086\000\085\000\
    \088\000\084\000\080\000\077\000\079\000\081\000\255\255\071\000\
    \255\255\072\000\072\000\072\000\072\000\072\000\072\000\072\000\
    \072\000\072\000\072\000\039\000\052\000\076\000\078\000\056\000\
    \057\000\082\000\083\000\104\000\095\000\052\000\049\000\038\000\
    \056\000\057\000\038\000\038\000\050\000\255\255\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \105\000\040\000\000\000\000\000\052\000\000\000\038\000\056\000\
    \057\000\000\000\000\000\051\000\000\000\052\000\000\000\000\000\
    \056\000\057\000\052\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\050\000\052\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\038\000\000\000\
    \000\000\000\000\046\000\051\000\000\000\041\000\255\255\103\000\
    \000\000\051\000\052\000\000\000\000\000\000\000\000\000\000\000\
    \052\000\000\000\000\000\052\000\000\000\000\000\000\000\000\000\
    \000\000\052\000\000\000\000\000\054\000\000\000\049\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\051\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \052\000\099\000\099\000\099\000\099\000\099\000\099\000\099\000\
    \099\000\052\000\000\000\000\000\054\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \040\000\000\000\000\000\000\000\033\000\000\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \043\000\255\255\000\000\000\000\255\255\044\000\037\000\037\000\
    \037\000\037\000\037\000\037\000\037\000\037\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \000\000\000\000\041\000\000\000\255\255\000\000\255\255\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\000\000\000\000\000\000\000\000\033\000\000\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\040\000\255\255\037\000\000\000\042\000\
    \000\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\000\000\000\000\000\000\
    \000\000\255\255\255\255\255\255\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\255\255\048\000\
    \047\000\255\255\042\000\000\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\048\000\061\000\
    \000\000\064\000\064\000\064\000\064\000\064\000\064\000\064\000\
    \064\000\064\000\064\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\046\000\046\000\046\000\063\000\062\000\
    \255\255\059\000\000\000\059\000\000\000\062\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \067\000\067\000\067\000\067\000\067\000\067\000\067\000\067\000\
    \067\000\067\000\000\000\046\000\000\000\000\000\063\000\062\000\
    \000\000\000\000\000\000\000\000\050\000\062\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\000\000\051\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\057\000\000\000\000\000\000\000\060\000\000\000\
    \070\000\000\000\070\000\057\000\060\000\069\000\069\000\069\000\
    \069\000\069\000\069\000\069\000\069\000\069\000\069\000\255\255\
    \000\000\000\000\255\255\051\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\057\000\000\000\000\000\000\000\060\000\000\000\
    \000\000\000\000\000\000\057\000\060\000\000\000\000\000\000\000\
    \041\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \068\000\000\000\068\000\000\000\000\000\067\000\067\000\067\000\
    \067\000\067\000\067\000\067\000\067\000\067\000\067\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\255\255\
    \255\255\255\255\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\037\000\103\000\000\000\000\000\037\000\000\000\
    \000\000\055\000\055\000\055\000\055\000\055\000\055\000\255\255\
    \000\000\000\000\000\000\000\000\056\000\000\000\000\000\000\000\
    \000\000\037\000\000\000\107\000\000\000\056\000\000\000\067\000\
    \067\000\067\000\067\000\067\000\067\000\067\000\067\000\067\000\
    \067\000\055\000\055\000\055\000\055\000\055\000\055\000\000\000\
    \000\000\000\000\000\000\000\000\056\000\065\000\000\000\000\000\
    \000\000\000\000\000\000\065\000\000\000\056\000\064\000\064\000\
    \064\000\064\000\064\000\064\000\064\000\064\000\064\000\064\000\
    \069\000\069\000\069\000\069\000\069\000\069\000\069\000\069\000\
    \069\000\069\000\000\000\066\000\065\000\065\000\000\000\000\000\
    \000\000\000\000\065\000\065\000\000\000\000\000\062\000\000\000\
    \000\000\000\000\000\000\000\000\062\000\069\000\069\000\069\000\
    \069\000\069\000\069\000\069\000\069\000\069\000\069\000\000\000\
    \000\000\000\000\000\000\066\000\065\000\000\000\000\000\000\000\
    \000\000\000\000\065\000\000\000\000\000\000\000\062\000\000\000\
    \000\000\000\000\000\000\000\000\062\000\072\000\072\000\072\000\
    \072\000\072\000\072\000\072\000\072\000\072\000\072\000\074\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\074\000\074\000\
    \074\000\000\000\073\000\065\000\000\000\075\000\255\255\075\000\
    \000\000\065\000\074\000\074\000\074\000\074\000\074\000\074\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\074\000\074\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\000\000\000\000\
    \000\000\000\000\073\000\065\000\000\000\000\000\041\000\000\000\
    \000\000\065\000\065\000\041\000\000\000\000\000\000\000\000\000\
    \065\000\000\000\000\000\000\000\096\000\096\000\096\000\096\000\
    \096\000\096\000\096\000\096\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\041\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\065\000\000\000\000\000\000\000\000\000\000\000\
    \065\000\102\000\102\000\102\000\102\000\102\000\102\000\102\000\
    \102\000\102\000\102\000\000\000\097\000\000\000\000\000\000\000\
    \041\000\255\255\000\000\000\000\000\000\041\000\041\000\000\000\
    \000\000\000\000\041\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\041\000\000\000\000\000\000\000\041\000\000\000\
    \041\000\000\000\041\000\000\000\097\000\098\000\098\000\098\000\
    \098\000\098\000\098\000\098\000\098\000\098\000\098\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\098\000\098\000\
    \098\000\098\000\098\000\098\000\098\000\098\000\098\000\098\000\
    \098\000\098\000\098\000\098\000\098\000\098\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\098\000\098\000\098\000\
    \098\000\098\000\098\000\037\000\103\000\000\000\098\000\098\000\
    \098\000\098\000\098\000\098\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\037\000\000\000\000\000\098\000\098\000\098\000\
    \098\000\098\000\098\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\102\000\102\000\102\000\102\000\102\000\102\000\
    \102\000\102\000\102\000\102\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\101\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\255\255";
  Lexing.lex_check = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\000\000\
    \000\000\010\000\010\000\013\000\016\000\019\000\022\000\023\000\
    \016\000\024\000\025\000\026\000\025\000\025\000\028\000\029\000\
    \030\000\029\000\029\000\029\000\029\000\029\000\029\000\029\000\
    \029\000\029\000\029\000\036\000\052\000\071\000\077\000\056\000\
    \057\000\081\000\082\000\101\000\093\000\052\000\028\000\038\000\
    \056\000\057\000\038\000\038\000\031\000\030\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \104\000\105\000\255\255\255\255\052\000\255\255\038\000\056\000\
    \057\000\255\255\255\255\031\000\255\255\052\000\255\255\255\255\
    \056\000\057\000\031\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\032\000\031\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\038\000\255\255\
    \255\255\255\255\030\000\031\000\255\255\039\000\043\000\109\000\
    \255\255\032\000\031\000\255\255\255\255\255\255\255\255\255\255\
    \032\000\255\255\255\255\031\000\255\255\255\255\255\255\255\255\
    \255\255\032\000\255\255\255\255\032\000\255\255\043\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\032\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \032\000\096\000\096\000\096\000\096\000\096\000\096\000\096\000\
    \096\000\032\000\255\255\255\255\032\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \093\000\255\255\255\255\255\255\033\000\255\255\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \034\000\107\000\255\255\255\255\107\000\034\000\099\000\099\000\
    \099\000\099\000\099\000\099\000\099\000\099\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \255\255\255\255\107\000\255\255\028\000\255\255\030\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\255\255\255\255\255\255\255\255\034\000\255\255\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\039\000\043\000\109\000\255\255\035\000\
    \255\255\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\255\255\255\255\255\255\
    \255\255\044\000\045\000\046\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\044\000\045\000\
    \046\000\047\000\042\000\255\255\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\047\000\050\000\
    \255\255\050\000\050\000\050\000\050\000\050\000\050\000\050\000\
    \050\000\050\000\050\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\044\000\045\000\046\000\050\000\050\000\
    \107\000\051\000\255\255\051\000\255\255\050\000\051\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \068\000\068\000\068\000\068\000\068\000\068\000\068\000\068\000\
    \068\000\068\000\255\255\047\000\255\255\255\255\050\000\050\000\
    \255\255\255\255\255\255\255\255\053\000\050\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\255\255\053\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\053\000\255\255\255\255\255\255\058\000\255\255\
    \063\000\255\255\063\000\053\000\058\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\108\000\
    \255\255\255\255\108\000\053\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\053\000\255\255\255\255\255\255\058\000\255\255\
    \255\255\255\255\255\255\053\000\058\000\255\255\255\255\255\255\
    \108\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\054\000\054\000\054\000\054\000\054\000\054\000\
    \066\000\255\255\066\000\255\255\255\255\066\000\066\000\066\000\
    \066\000\066\000\066\000\066\000\066\000\066\000\066\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\044\000\
    \045\000\046\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\106\000\106\000\255\255\255\255\106\000\255\255\
    \255\255\055\000\055\000\055\000\055\000\055\000\055\000\047\000\
    \255\255\255\255\255\255\255\255\055\000\255\255\255\255\255\255\
    \255\255\106\000\255\255\106\000\255\255\055\000\255\255\067\000\
    \067\000\067\000\067\000\067\000\067\000\067\000\067\000\067\000\
    \067\000\055\000\055\000\055\000\055\000\055\000\055\000\255\255\
    \255\255\255\255\255\255\255\255\055\000\067\000\255\255\255\255\
    \255\255\255\255\255\255\067\000\255\255\055\000\064\000\064\000\
    \064\000\064\000\064\000\064\000\064\000\064\000\064\000\064\000\
    \069\000\069\000\069\000\069\000\069\000\069\000\069\000\069\000\
    \069\000\069\000\255\255\064\000\064\000\067\000\255\255\255\255\
    \255\255\255\255\064\000\067\000\255\255\255\255\069\000\255\255\
    \255\255\255\255\255\255\255\255\069\000\070\000\070\000\070\000\
    \070\000\070\000\070\000\070\000\070\000\070\000\070\000\255\255\
    \255\255\255\255\255\255\064\000\064\000\255\255\255\255\255\255\
    \255\255\255\255\064\000\255\255\255\255\255\255\069\000\255\255\
    \255\255\255\255\255\255\255\255\069\000\072\000\072\000\072\000\
    \072\000\072\000\072\000\072\000\072\000\072\000\072\000\075\000\
    \075\000\075\000\075\000\075\000\075\000\075\000\075\000\075\000\
    \075\000\255\255\072\000\072\000\255\255\073\000\108\000\073\000\
    \255\255\072\000\073\000\073\000\073\000\073\000\073\000\073\000\
    \073\000\073\000\073\000\073\000\074\000\074\000\074\000\074\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\255\255\255\255\
    \255\255\255\255\072\000\072\000\255\255\255\255\095\000\255\255\
    \255\255\072\000\074\000\095\000\255\255\255\255\255\255\255\255\
    \074\000\255\255\255\255\255\255\095\000\095\000\095\000\095\000\
    \095\000\095\000\095\000\095\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\095\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\074\000\255\255\255\255\255\255\255\255\255\255\
    \074\000\102\000\102\000\102\000\102\000\102\000\102\000\102\000\
    \102\000\102\000\102\000\255\255\095\000\255\255\255\255\255\255\
    \095\000\106\000\255\255\255\255\255\255\095\000\095\000\255\255\
    \255\255\255\255\095\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\095\000\255\255\255\255\255\255\095\000\255\255\
    \095\000\255\255\095\000\255\255\095\000\097\000\097\000\097\000\
    \097\000\097\000\097\000\097\000\097\000\097\000\097\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\097\000\097\000\
    \097\000\097\000\097\000\097\000\098\000\098\000\098\000\098\000\
    \098\000\098\000\098\000\098\000\098\000\098\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\098\000\098\000\098\000\
    \098\000\098\000\098\000\100\000\100\000\255\255\097\000\097\000\
    \097\000\097\000\097\000\097\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\100\000\255\255\255\255\098\000\098\000\098\000\
    \098\000\098\000\098\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\100\000\100\000\100\000\100\000\100\000\100\000\
    \100\000\100\000\100\000\100\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\100\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\100\000";
  Lexing.lex_base_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\001\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \004\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_backtrk_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\004\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_default_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_trans_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\001\000\
    \001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000";
  Lexing.lex_check_code = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\000\000\
    \034\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255";
  Lexing.lex_code = 
   "\255\001\255\255\000\001\255";
}

let rec token lexbuf =
  lexbuf.Lexing.lex_mem <- Array.create 2 (-1) ;   __ocaml_lex_token_rec lexbuf 0
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.new_engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 236 "cil/src/logic/logic_lexer.mll"
           ( token lexbuf )
# 738 "cil/src/logic/logic_lexer.ml"

  | 1 ->
# 237 "cil/src/logic/logic_lexer.mll"
         ( update_newline_loc lexbuf; token lexbuf )
# 743 "cil/src/logic/logic_lexer.ml"

  | 2 ->
# 238 "cil/src/logic/logic_lexer.mll"
                      ( update_newline_loc lexbuf; token lexbuf )
# 748 "cil/src/logic/logic_lexer.ml"

  | 3 ->
# 239 "cil/src/logic/logic_lexer.mll"
                     ( token lexbuf )
# 753 "cil/src/logic/logic_lexer.ml"

  | 4 ->
# 241 "cil/src/logic/logic_lexer.mll"
                       ( bs_identifier lexbuf )
# 758 "cil/src/logic/logic_lexer.ml"

  | 5 ->
# 242 "cil/src/logic/logic_lexer.mll"
                        ( let s = lexeme lexbuf in identifier s )
# 763 "cil/src/logic/logic_lexer.ml"

  | 6 ->
# 244 "cil/src/logic/logic_lexer.mll"
                            ( CONSTANT (IntConstant (lexeme lexbuf)) )
# 768 "cil/src/logic/logic_lexer.ml"

  | 7 ->
# 245 "cil/src/logic/logic_lexer.mll"
                            ( CONSTANT (IntConstant (lexeme lexbuf)) )
# 773 "cil/src/logic/logic_lexer.ml"

  | 8 ->
# 246 "cil/src/logic/logic_lexer.mll"
                            ( CONSTANT (IntConstant (lexeme lexbuf)) )
# 778 "cil/src/logic/logic_lexer.ml"

  | 9 ->
let
# 247 "cil/src/logic/logic_lexer.mll"
                 prelude
# 784 "cil/src/logic/logic_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_mem.(0)
and
# 247 "cil/src/logic/logic_lexer.mll"
                                                   content
# 789 "cil/src/logic/logic_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) (lexbuf.Lexing.lex_curr_pos + -1) in
# 248 "cil/src/logic/logic_lexer.mll"
      (
        let b = Buffer.create 5 in
        Buffer.add_string b prelude;
        let lbf = Lexing.from_string content in
        chr b lbf
      )
# 798 "cil/src/logic/logic_lexer.ml"

  | 10 ->
# 254 "cil/src/logic/logic_lexer.mll"
                            ( CONSTANT (FloatConstant (lexeme lexbuf)) )
# 803 "cil/src/logic/logic_lexer.ml"

  | 11 ->
# 255 "cil/src/logic/logic_lexer.mll"
                            ( CONSTANT (FloatConstant (lexeme lexbuf)) )
# 808 "cil/src/logic/logic_lexer.ml"

  | 12 ->
# 256 "cil/src/logic/logic_lexer.mll"
                            ( CONSTANT (FloatConstant (lexeme lexbuf)) )
# 813 "cil/src/logic/logic_lexer.ml"

  | 13 ->
let
# 259 "cil/src/logic/logic_lexer.mll"
            n
# 819 "cil/src/logic/logic_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_curr_pos + -2) in
# 259 "cil/src/logic/logic_lexer.mll"
                            ( lexbuf.lex_curr_pos <- lexbuf.lex_curr_pos - 2;
                              CONSTANT (IntConstant n) )
# 824 "cil/src/logic/logic_lexer.ml"

  | 14 ->
# 262 "cil/src/logic/logic_lexer.mll"
                                ( STRING_LITERAL (lexeme lexbuf) )
# 829 "cil/src/logic/logic_lexer.ml"

  | 15 ->
# 264 "cil/src/logic/logic_lexer.mll"
                            ( hash lexbuf )
# 834 "cil/src/logic/logic_lexer.ml"

  | 16 ->
# 265 "cil/src/logic/logic_lexer.mll"
                            ( IMPLIES )
# 839 "cil/src/logic/logic_lexer.ml"

  | 17 ->
# 266 "cil/src/logic/logic_lexer.mll"
                            ( IFF )
# 844 "cil/src/logic/logic_lexer.ml"

  | 18 ->
# 267 "cil/src/logic/logic_lexer.mll"
                            ( AND )
# 849 "cil/src/logic/logic_lexer.ml"

  | 19 ->
# 268 "cil/src/logic/logic_lexer.mll"
                            ( OR )
# 854 "cil/src/logic/logic_lexer.ml"

  | 20 ->
# 269 "cil/src/logic/logic_lexer.mll"
                            ( NOT )
# 859 "cil/src/logic/logic_lexer.ml"

  | 21 ->
# 270 "cil/src/logic/logic_lexer.mll"
                            ( DOLLAR )
# 864 "cil/src/logic/logic_lexer.ml"

  | 22 ->
# 271 "cil/src/logic/logic_lexer.mll"
                            ( COMMA )
# 869 "cil/src/logic/logic_lexer.ml"

  | 23 ->
# 272 "cil/src/logic/logic_lexer.mll"
                            ( ARROW )
# 874 "cil/src/logic/logic_lexer.ml"

  | 24 ->
# 273 "cil/src/logic/logic_lexer.mll"
                            ( Stack.push Test state_stack; QUESTION )
# 879 "cil/src/logic/logic_lexer.ml"

  | 25 ->
# 274 "cil/src/logic/logic_lexer.mll"
                            ( SEMICOLON )
# 884 "cil/src/logic/logic_lexer.ml"

  | 26 ->
# 275 "cil/src/logic/logic_lexer.mll"
                            ( match get_state() with
                                  Normal  -> COLON
                                | Test -> pop_state(); COLON2
                            )
# 892 "cil/src/logic/logic_lexer.ml"

  | 27 ->
# 279 "cil/src/logic/logic_lexer.mll"
                            ( COLONCOLON )
# 897 "cil/src/logic/logic_lexer.ml"

  | 28 ->
# 280 "cil/src/logic/logic_lexer.mll"
                            ( DOT )
# 902 "cil/src/logic/logic_lexer.ml"

  | 29 ->
# 281 "cil/src/logic/logic_lexer.mll"
                            ( DOTDOT )
# 907 "cil/src/logic/logic_lexer.ml"

  | 30 ->
# 282 "cil/src/logic/logic_lexer.mll"
                            ( DOTDOTDOT )
# 912 "cil/src/logic/logic_lexer.ml"

  | 31 ->
# 283 "cil/src/logic/logic_lexer.mll"
                            ( MINUS )
# 917 "cil/src/logic/logic_lexer.ml"

  | 32 ->
# 284 "cil/src/logic/logic_lexer.mll"
                            ( PLUS )
# 922 "cil/src/logic/logic_lexer.ml"

  | 33 ->
# 285 "cil/src/logic/logic_lexer.mll"
                            ( STAR )
# 927 "cil/src/logic/logic_lexer.ml"

  | 34 ->
# 286 "cil/src/logic/logic_lexer.mll"
                            ( AMP )
# 932 "cil/src/logic/logic_lexer.ml"

  | 35 ->
# 287 "cil/src/logic/logic_lexer.mll"
                            ( HATHAT )
# 937 "cil/src/logic/logic_lexer.ml"

  | 36 ->
# 288 "cil/src/logic/logic_lexer.mll"
                            ( HAT )
# 942 "cil/src/logic/logic_lexer.ml"

  | 37 ->
# 289 "cil/src/logic/logic_lexer.mll"
                            ( PIPE )
# 947 "cil/src/logic/logic_lexer.ml"

  | 38 ->
# 290 "cil/src/logic/logic_lexer.mll"
                            ( TILDE )
# 952 "cil/src/logic/logic_lexer.ml"

  | 39 ->
# 291 "cil/src/logic/logic_lexer.mll"
                            ( SLASH )
# 957 "cil/src/logic/logic_lexer.ml"

  | 40 ->
# 292 "cil/src/logic/logic_lexer.mll"
                            ( PERCENT )
# 962 "cil/src/logic/logic_lexer.ml"

  | 41 ->
# 293 "cil/src/logic/logic_lexer.mll"
                            ( LT )
# 967 "cil/src/logic/logic_lexer.ml"

  | 42 ->
# 294 "cil/src/logic/logic_lexer.mll"
                            ( GT )
# 972 "cil/src/logic/logic_lexer.ml"

  | 43 ->
# 295 "cil/src/logic/logic_lexer.mll"
                            ( LE )
# 977 "cil/src/logic/logic_lexer.ml"

  | 44 ->
# 296 "cil/src/logic/logic_lexer.mll"
                            ( GE )
# 982 "cil/src/logic/logic_lexer.ml"

  | 45 ->
# 297 "cil/src/logic/logic_lexer.mll"
                            ( EQ )
# 987 "cil/src/logic/logic_lexer.ml"

  | 46 ->
# 298 "cil/src/logic/logic_lexer.mll"
                            ( EQUAL )
# 992 "cil/src/logic/logic_lexer.ml"

  | 47 ->
# 299 "cil/src/logic/logic_lexer.mll"
                            ( NE )
# 997 "cil/src/logic/logic_lexer.ml"

  | 48 ->
# 300 "cil/src/logic/logic_lexer.mll"
                            ( Stack.push Normal state_stack; LPAR )
# 1002 "cil/src/logic/logic_lexer.ml"

  | 49 ->
# 301 "cil/src/logic/logic_lexer.mll"
                            ( pop_state(); RPAR )
# 1007 "cil/src/logic/logic_lexer.ml"

  | 50 ->
# 302 "cil/src/logic/logic_lexer.mll"
                            ( Stack.push Normal state_stack; LBRACE )
# 1012 "cil/src/logic/logic_lexer.ml"

  | 51 ->
# 303 "cil/src/logic/logic_lexer.mll"
                            ( pop_state(); RBRACE )
# 1017 "cil/src/logic/logic_lexer.ml"

  | 52 ->
# 304 "cil/src/logic/logic_lexer.mll"
                            ( Stack.push Normal state_stack; LSQUARE )
# 1022 "cil/src/logic/logic_lexer.ml"

  | 53 ->
# 305 "cil/src/logic/logic_lexer.mll"
                            ( pop_state(); RSQUARE )
# 1027 "cil/src/logic/logic_lexer.ml"

  | 54 ->
# 306 "cil/src/logic/logic_lexer.mll"
                            ( LTCOLON )
# 1032 "cil/src/logic/logic_lexer.ml"

  | 55 ->
# 307 "cil/src/logic/logic_lexer.mll"
                            ( COLONGT )
# 1037 "cil/src/logic/logic_lexer.ml"

  | 56 ->
# 308 "cil/src/logic/logic_lexer.mll"
                            ( LTLT )
# 1042 "cil/src/logic/logic_lexer.ml"

  | 57 ->
# 309 "cil/src/logic/logic_lexer.mll"
                            ( GTGT )
# 1047 "cil/src/logic/logic_lexer.ml"

  | 58 ->
let
# 310 "cil/src/logic/logic_lexer.mll"
                 c
# 1053 "cil/src/logic/logic_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 310 "cil/src/logic/logic_lexer.mll"
                            ( find_utf8 c )
# 1057 "cil/src/logic/logic_lexer.ml"

  | 59 ->
# 311 "cil/src/logic/logic_lexer.mll"
                            ( EOF )
# 1062 "cil/src/logic/logic_lexer.ml"

  | 60 ->
# 312 "cil/src/logic/logic_lexer.mll"
        ( lex_error lexbuf ("illegal character " ^ lexeme lexbuf) )
# 1067 "cil/src/logic/logic_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_token_rec lexbuf __ocaml_lex_state

and chr buffer lexbuf =
    __ocaml_lex_chr_rec buffer lexbuf 93
and __ocaml_lex_chr_rec buffer lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 316 "cil/src/logic/logic_lexer.mll"
      ( let s = lexeme lexbuf in
        let real_s = String.sub s 2 (String.length s - 2) in
        let rec add_one_char s =
          let l = String.length s in
          if l = 0 then ()
          else
          let h = int_of_digit s.[0] in
          let c,s =
            if l = 1 then (h,"")
            else
              (16*h + int_of_digit s.[1],
               String.sub s 2 (String.length s - 2))
          in
          Buffer.add_char buffer (Char.chr c); add_one_char s
        in add_one_char real_s; chr buffer lexbuf
      )
# 1093 "cil/src/logic/logic_lexer.ml"

  | 1 ->
# 333 "cil/src/logic/logic_lexer.mll"
      ( let s = lexeme lexbuf in
        let real_s = String.sub s 1 (String.length s - 1) in
        let rec value i s =
          if s = "" then i
          else value (8*i+int_of_digit s.[0])
            (String.sub s 1 (String.length s -1))
        in let c = value 0 real_s in
        Buffer.add_char buffer (Char.chr c); chr buffer lexbuf
      )
# 1106 "cil/src/logic/logic_lexer.ml"

  | 2 ->
# 343 "cil/src/logic/logic_lexer.mll"
      ( Buffer.add_char buffer
          (match (lexeme lexbuf).[1] with
               'a' -> '\007'
             | 'b' -> '\b'
             | 'f' -> '\012'
             | 'n' -> '\n'
             | 'r' -> '\r'
             | 't' -> '\t'
             | '\'' -> '\''
             | '"' -> '"'
             | '?' -> '?'
             | '\\' -> '\\'
             | _ -> assert false
          ); chr buffer lexbuf)
# 1124 "cil/src/logic/logic_lexer.ml"

  | 3 ->
# 357 "cil/src/logic/logic_lexer.mll"
        ( Buffer.add_char buffer '\'';
          CONSTANT (IntConstant (Buffer.contents buffer)) )
# 1130 "cil/src/logic/logic_lexer.ml"

  | 4 ->
# 359 "cil/src/logic/logic_lexer.mll"
       ( Buffer.add_string buffer (lexeme lexbuf); chr buffer lexbuf )
# 1135 "cil/src/logic/logic_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_chr_rec buffer lexbuf __ocaml_lex_state

and hash lexbuf =
    __ocaml_lex_hash_rec lexbuf 100
and __ocaml_lex_hash_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 362 "cil/src/logic/logic_lexer.mll"
        ( update_newline_loc lexbuf; token lexbuf)
# 1146 "cil/src/logic/logic_lexer.ml"

  | 1 ->
# 363 "cil/src/logic/logic_lexer.mll"
             ( hash lexbuf)
# 1151 "cil/src/logic/logic_lexer.ml"

  | 2 ->
# 364 "cil/src/logic/logic_lexer.mll"
              ( (* We are seeing a line number. This is the number for the
                   * next line *)
                 let s = Lexing.lexeme lexbuf in
                 let lineno = try
                   int_of_string s
                 with Failure ("int_of_string") ->
                   (* the int is too big. *)
                   Cil.warnLoc (lexbuf.lex_start_p, lexbuf.lex_curr_p)
                     "Bad line number in preprocessed file: %s"  s;
                   (-1)
                 in
                 update_line_loc lexbuf (lineno - 1) true 0;
                  (* A file name may follow *)
		  file lexbuf )
# 1169 "cil/src/logic/logic_lexer.ml"

  | 3 ->
# 378 "cil/src/logic/logic_lexer.mll"
                ( hash lexbuf )
# 1174 "cil/src/logic/logic_lexer.ml"

  | 4 ->
# 379 "cil/src/logic/logic_lexer.mll"
            ( endline lexbuf)
# 1179 "cil/src/logic/logic_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_hash_rec lexbuf __ocaml_lex_state

and file lexbuf =
    __ocaml_lex_file_rec lexbuf 106
and __ocaml_lex_file_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 382 "cil/src/logic/logic_lexer.mll"
                      ( update_newline_loc lexbuf; token lexbuf)
# 1190 "cil/src/logic/logic_lexer.ml"

  | 1 ->
# 383 "cil/src/logic/logic_lexer.mll"
                  ( file lexbuf)
# 1195 "cil/src/logic/logic_lexer.ml"

  | 2 ->
# 385 "cil/src/logic/logic_lexer.mll"
            (
              let n = Lexing.lexeme lexbuf in
              let n1 = String.sub n 1
                ((String.length n) - 2) in
              update_file_loc lexbuf n1;
	      endline lexbuf
            )
# 1206 "cil/src/logic/logic_lexer.ml"

  | 3 ->
# 393 "cil/src/logic/logic_lexer.mll"
      ( endline lexbuf)
# 1211 "cil/src/logic/logic_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_file_rec lexbuf __ocaml_lex_state

and endline lexbuf =
    __ocaml_lex_endline_rec lexbuf 109
and __ocaml_lex_endline_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 396 "cil/src/logic/logic_lexer.mll"
                ( update_newline_loc lexbuf; token lexbuf)
# 1222 "cil/src/logic/logic_lexer.ml"

  | 1 ->
# 397 "cil/src/logic/logic_lexer.mll"
                                ( EOF )
# 1227 "cil/src/logic/logic_lexer.ml"

  | 2 ->
# 398 "cil/src/logic/logic_lexer.mll"
      ( endline lexbuf)
# 1232 "cil/src/logic/logic_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_endline_rec lexbuf __ocaml_lex_state

;;

# 400 "cil/src/logic/logic_lexer.mll"
 

  open Format

  let copy_lexbuf dest_lexbuf src_loc =
    update_file_loc dest_lexbuf src_loc.pos_fname;
    update_line_loc dest_lexbuf src_loc.pos_lnum true 0;
    let bol = src_loc.Lexing.pos_cnum - src_loc.Lexing.pos_bol in
    update_bol_loc dest_lexbuf (-bol)

  let start_pos lexbuf =
    let pos = lexeme_start_p lexbuf in
    pos.Lexing.pos_cnum - pos.Lexing.pos_bol

  let end_pos lexbuf =
    let pos = lexeme_end_p lexbuf in
    pos.Lexing.pos_cnum - pos.Lexing.pos_bol

  let parse_from_location f (loc, s : Lexing.position * string) =
    let lb = from_string s in
    copy_lexbuf lb loc;
    try
      f token lb
    with
      | Parsing.Parse_error as _e ->
          let start, stop = start_pos lb,end_pos lb in
          Cil.error_loc (
	    lb.lex_curr_p.Lexing.pos_fname,
	    lb.lex_curr_p.Lexing.pos_lnum,start,stop)
            "syntax error while parsing annotation@.";
          Logic_const.exit_kw_c_mode ();
          raise Parsing.Parse_error

      | Error ((b,e), m) ->
          Cil.error_loc (
	    lb.lex_curr_p.Lexing.pos_fname,
	    lb.lex_curr_p.Lexing.pos_lnum,b,e)
            "%s@."
            m;
          Logic_const.exit_kw_c_mode ();
          raise Parsing.Parse_error
      | Logic_const.Not_well_formed (loc, m) ->
          Cil.error_loc
            ((fst loc).Lexing.pos_fname,
             (fst loc).Lexing.pos_lnum,
             (fst loc).Lexing.pos_cnum - (fst loc).Lexing.pos_bol,
             (snd loc).Lexing.pos_cnum - (snd loc).Lexing.pos_bol)
            "%s@." m;
          Logic_const.exit_kw_c_mode ();
          raise Parsing.Parse_error


  let lexpr = parse_from_location Logic_parser.lexpr

  let annot = parse_from_location Logic_parser.annot

  let spec = parse_from_location Logic_parser.spec


# 1298 "cil/src/logic/logic_lexer.ml"
