/**************************************************************************/
/*                                                                        */
/*  This file is part of Frama-C.                                         */
/*                                                                        */
/*  Copyright (C) 2007-2008                                               */
/*    INRIA (Institut National de Recherche en Informatique et en         */
/*           Automatique)                                                 */
/*                                                                        */
/*  you can redistribute it and/or modify it under the terms of the GNU   */
/*  Lesser General Public License as published by the Free Software       */
/*  Foundation, version 2.1.                                              */
/*                                                                        */
/*  It is distributed in the hope that it will be useful,                 */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of        */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         */
/*  GNU Lesser General Public License for more details.                   */
/*                                                                        */
/*  See the GNU Lesser General Public License version 2.1                 */
/*  for more details (enclosed in the file licenses/LGPLv2.1).            */
/**************************************************************************/

/* $Id: stdlib.h,v 1.5 2008/10/28 16:16:11 uid570 Exp $ */

#ifndef _STDLIB_H_
#define _STDLIB_H_

#include <stddef.h>
#include <string.h>
#include <wchar.h>

// Conversion functions

/*@ requires valid_string(nptr);
  @ assigns \nothing;
  @*/
extern double atof(const char *nptr);

/*@ requires valid_string(nptr);
  @ assigns \nothing;
  @*/
extern int atoi(const char *nptr);

/*@ requires valid_string(nptr);
  @ assigns \nothing;
  @*/
extern long int atol(const char *nptr);

/*@ requires valid_string(nptr);
  @ assigns \nothing;
  @*/
extern long long int atoll(const char *nptr);

// Random numbers

extern int rand(void);

extern void srand(unsigned int seed);

// Memory management

/*@ assigns \nothing;
  @ ensures \valid_range((char*)\result,0,size - 1);
  @*/
extern void *malloc (size_t size);

/*@ assigns \nothing;
  @ ensures \valid_range((char*)\result,0,size * nmemb - 1);
  @*/
extern void *calloc (size_t nmemb, size_t size);

/*@ requires \valid((char*)ptr);
  @ assigns \nothing;
  @ ensures \valid_range((char*)\result,0,size - 1);
  @*/
extern void *realloc (void *ptr, size_t size);

/*@ requires \valid((char*)ptr);
  @ assigns \nothing;
  @ ensures ! \valid((char*)ptr);
  @*/
extern void free (void *ptr);

// Program termination

//@ ensures \false;
extern void abort(void);

extern int atexit(void (*func)(void));

//@ ensures \false;
extern void exit(int status);

// Multi-byte conversion functions

/*@ requires valid_string(mbstr)
  @          && \valid_range(wcstr,0,max - 1);
  @ assigns wcstr[0..max - 1];
  @ ensures valid_wstring(wcstr);
  @*/
extern size_t mbstowcs (wchar_t *wcstr, const char *mbstr, size_t max);

#endif /* _STDLIB_H_ */
