(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2008                                               *)
(*    CEA (Commissariat  l'nergie Atomique)                             *)
(*                                                                        *)
(*  All rights reserved.                                                  *)
(*  Contact CEA for more details about the license.                       *)
(*                                                                        *)
(**************************************************************************)

(* $Id: register_gui.ml,v 1.2 2008/10/03 13:09:16 uid568 Exp $ *)

open Pretty_source
open Gtk_helper
open Db
open Cil_types

module HighlightedStmtState = struct

  include Cil_computation.StmtSetRef
    (struct
       let name = "Impact_gui.HighlightedStmt"
       let dependencies = [ Cil_state.self ]
     end)

  let clear () = 
    let only = 
      Project.Selection.singleton self Kind.Select_Dependencies
    in
    Project.clear ~only ()

end

let impact_highlighter buffer loc ~start ~stop =
  let tag = make_tag buffer "impact" [`BACKGROUND "green" ] in
  match loc with
  | PStmt (_,stmt) -> 
      if HighlightedStmtState.mem stmt then 
        apply_tag buffer tag start stop
  | _ -> ()
 
let impact_selector
    (popup_factory:GMenu.menu GMenu.factory) main_ui ~button localizable =
  if button = 3 then
    match localizable with
    | PStmt (_kf,ki) ->
        ignore
          (popup_factory#add_item "_Impact analysis"
             ~callback:
             (fun () ->
                main_ui#lock ();
                HighlightedStmtState.clear ();
                List.iter HighlightedStmtState.add (!Db.Impact.from_stmt ki);
		main_ui#rehighlight ();
		main_ui#unlock ()));
    | _ ->
	()
	  
let main main_ui = 
  main_ui#register_source_selector impact_selector;
  main_ui#register_source_highlighter impact_highlighter  
  
let () = 
  Design.register_extension main;
  Design.register_reset_extension (fun _ -> HighlightedStmtState.clear ())

(*
Local Variables:
compile-command: "LC_ALL=C make -C ../.. -j"
End:
*)
