(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2008                                               *)
(*    CEA (Commissariat  l'nergie Atomique)                             *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* $Id: funTbl.mli,v 1.5 2008/08/27 12:28:15 uid568 Exp $ *)

type t

(** raise by apply if the given [type] is not the same as previously
    registered by [register] *)
exception Not_Registered of string
  
(** raise by apply if the given name is not registered in [tbl] *)
exception Incompatible_Type of string
  
(** raise by registerf the given name is already registered in [tbl] *)
exception AlreadyExists of string

(** [create n] create  a table of functions with n cases*)
val create: int -> t

(** [register tbl name type function] registered the [function] in [tbl] with
    the given [name] and the given [type].
    @raise AlreadyExists if the given name is already registered in [tbl] *)
val register: t -> string -> 'a Type.t -> 'a -> unit 

(** [appply tbl name type] return the function registered in [tbl] with the
    given [name] and the given [type].
    @raise Incompatible_Type if the given [type] is not the same as previously
    registered by [register]
    @raise Not_Registered if the given name is not registered in [tbl] *) 
val apply : t -> string -> 'a Type.t -> 'a

(**/**)
(** Same as apply but this function is very unsafe. 
    This function is not for the casual user.
    @raise Not_Registered if the given name is not registered in [tbl] *)
val unsafe_apply : t -> string -> 'a
(**/**)

(*
Local Variables:
compile-command: "LC_ALL=C make -C ../.. -j"
End:
*)
